<?php

namespace Drupal\entity_workflow\Controller;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\entity_workflow\EntityWorkflowInfo;
use Drupal\entity_workflow\Form\EntityWorkflowBulkTransitionForm;
use Drupal\entity_workflow\Form\EntityWorkflowForm;
use Drupal\workspaces\WorkspaceAssociationInterface;
use Drupal\workspaces\WorkspaceManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Provides responses and custom access checks for entity workflow routes.
 */
class EntityWorkflowController extends ControllerBase {

  /**
   * The workspace manager.
   *
   * @var \Drupal\workspaces\WorkspaceManagerInterface
   */
  protected $workspaceManager;

  /**
   * The workspace association service.
   *
   * @var \Drupal\workspaces\WorkspaceAssociationInterface
   */
  protected $workspaceAssociation;

  /**
   * The entity workflow information service.
   *
   * @var \Drupal\entity_workflow\EntityWorkflowInfo
   */
  protected $entityWorkflowInfo;

  /**
   * Constructs a new EntityWorkflowController instance.
   *
   * @param \Drupal\workspaces\WorkspaceManagerInterface $workspace_manager
   *   The workspace manager service.
   * @param \Drupal\workspaces\WorkspaceAssociationInterface $workspace_association
   *   The workspace association service.
   * @param \Drupal\entity_workflow\EntityWorkflowInfo $entity_workflow_info
   *   The entity workflow information service.
   */
  public function __construct(WorkspaceManagerInterface $workspace_manager, WorkspaceAssociationInterface $workspace_association, EntityWorkflowInfo $entity_workflow_info) {
    $this->workspaceManager = $workspace_manager;
    $this->workspaceAssociation = $workspace_association;
    $this->entityWorkflowInfo = $entity_workflow_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('workspaces.manager'),
      $container->get('workspaces.association'),
      $container->get('entity_workflow.info')
    );
  }

  /**
   * Builds and processes the workflow form.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   The workflow form.
   */
  public function workflow(RouteMatchInterface $route_match) {
    $entity = $route_match->getParameter($route_match->getParameter('entity_type_id'));
    $workflow = $route_match->getParameter('workflow');

    $build['transition'] = $this->formBuilder()->getForm(EntityWorkflowForm::class, $workflow, $entity);

    $build['history'] = [
      '#type' => 'view',
      '#name' => 'workflow_transition_log',
      '#display_id' => 'default',
      '#arguments' => [$entity->getEntityTypeId(), $entity->id()],
      '#embed' => TRUE,
    ];

    return $build;
  }

  public function checkWorkflowAccess(RouteMatchInterface $route_match) {
    $entity = $route_match->getParameter($route_match->getParameter('entity_type_id'));
    $workflow_id = $route_match->getRawParameter('workflow');

    return AccessResult::allowedIf($this->entityWorkflowInfo->isEntityInWorkflow($entity, $workflow_id))
      ->addCacheableDependency($entity);
  }

  /**
   * Builds and processes the bulk workflow form.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   The bulk workflow form.
   */
  public function bulkWorkflow(RouteMatchInterface $route_match) {
    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $route_match->getParameter($route_match->getParameter('entity_type_id'));
    $workflow = $route_match->getParameter('workflow');

    // Reconfigure changed entities into the format needed.
    $entities = $this->moduleHandler()->invokeAll('entity_workflow_bulk_workflow_entities', [$workflow, $entity]);

    // We didn't fetch all entities in access because it's potentially
    // expensive.
    if (!$entities) {
      throw new NotFoundHttpException();
    }

    // Add the current entity as the top one, regardless of what anyone else
    // said.
    $entities = [$entity->getEntityTypeId() . '--' . $entity->id() => $entity] + $entities;
    $this->moduleHandler()->alter('entity_workflow_bulk_workflow_entities', $entities, $workflow, $entity);

    // Access will already have failed if this isn't valid.
    $workspace = $workspace ?? $this->workspaceManager->getActiveWorkspace();
    return $this->formBuilder()->getForm(EntityWorkflowBulkTransitionForm::class, $workflow, $entities, $workspace);
  }

  /**
   * Checks access for the bulk workflow form.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function checkBulkWorkflowAccess(RouteMatchInterface $route_match) {
    if (!$this->checkWorkflowAccess($route_match)->isAllowed()) {
      return AccessResult::forbidden();
    }

    $workflow = $route_match->getParameter('workflow');
    $entity = $route_match->getParameter($route_match->getParameter('entity_type_id'));

    $access = FALSE;
    $this->moduleHandler()->alter('entity_workflow_has_bulk_workflow', $access, $workflow, $entity);
    return AccessResult::allowedIf($access);
  }

  /**
   * Builds and processes the bulk workflow form for a workspace.
   *
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The route match.
   *
   * @return array|\Symfony\Component\HttpFoundation\RedirectResponse
   *   The bulk workflow workspace form.
   */
  public function bulkWorkflowWorkspace(RouteMatchInterface $route_match) {
    $workspace = $route_match->getParameter('workspace');
    $workflow = $route_match->getParameter('workflow');

    $entities = [];
    foreach ($this->workspaceAssociation->getTrackedEntities($workspace->id()) as $entity_type_id => $entity_ids) {
      $tracked_entities = $this->entityTypeManager()->getStorage($entity_type_id)->loadMultiple($entity_ids);
      foreach ($tracked_entities as $entity_id => $entity) {
        $entities[$entity_type_id . '--' . $entity_id] = $entity;
      }
    }

    return $this->formBuilder()->getForm(EntityWorkflowBulkTransitionForm::class, $workflow, $entities, $workspace);
  }

  /**
   * Checks access for the bulk workflow workspace form.
   *
   * @return \Drupal\Core\Access\AccessResultInterface
   *   The access result.
   */
  public function checkBulkWorkflowWorkspaceAccess(RouteMatchInterface $route_match) {
    /** @var \Drupal\workspaces\WorkspaceInterface $workspace */
    $workspace = $route_match->getParameter('workspace');

    return $workspace->access('update', NULL, TRUE);
  }

}
