<?php

namespace Drupal\entity_workflow\Entity;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\entity_workflow\WorkflowTransitionLogInterface;

/**
 * Defines the 'workflow_transition_log' entity class.
 *
 * @ContentEntityType(
 *   id = "workflow_transition_log",
 *   label = @Translation("Workflow Transition Log"),
 *   label_singular = @Translation("transition log"),
 *   label_plural = @Translation("transition logs"),
 *   label_count = @PluralTranslation(
 *     singular = "@count transition log",
 *     plural = "@count transition logs",
 *   ),
 *   handlers = {
 *     "storage" = "Drupal\Core\Entity\Sql\SqlContentEntityStorage",
 *     "storage_schema" = "Drupal\entity_workflow\WorkflowTransitionLogStorageSchema",
 *     "views_data" = "\Drupal\views\EntityViewsData",
 *   },
 *   base_table = "workflow_transition_log",
 *   internal = TRUE,
 *   entity_keys = {
 *     "id" = "log_id",
 *   },
 * )
 */
class WorkflowTransitionLog extends ContentEntityBase implements WorkflowTransitionLogInterface {

  /**
   * Constructs a new transition log entity, without permanently saving it.
   */
  public static function createFromEntity(ContentEntityInterface $entity, $workflow_id, $previous_state, $new_state, $transition_id, $log) {
    $id_field = $entity->getFieldDefinition($entity->getEntityType()->getKey('id'));
    $source_entity_id = $id_field->getType() === 'integer' ? 'entity_id' : 'entity_id_string';
    if ($entity->getEntityTypeId() === 'workspace') {
      $workspace_id = $entity->id();
    }
    else {
      $field_name = $entity->getEntityType()->getRevisionMetadataKey('workspace');
      $workspace_id = !$entity->get($field_name)->isEmpty() ? $entity->get($field_name)->target_id : \Drupal::service('workspaces.manager')->getActiveWorkspace()->id();
    }

    return static::create([
      'entity_type_id' => $entity->getEntityTypeId(),
      $source_entity_id => $entity->id(),
      'entity_revision_id' => $entity->getRevisionId(),
      'workflow_id' => $workflow_id,
      'workspace_id' => $workspace_id,
      'state' => $new_state,
      'transition' => $transition_id,
      'previous_state' => $previous_state,
      'log' => $log,
    ]);
  }

  /**
   * {@inheritdoc}
   */
  public function getWorkflowId() {
    return $this->get('workflow_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function getWorkflow() {
    return $this->get('workflow_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceEntityId() {
    if (!$this->get('entity_id')->isEmpty()) {
      return $this->get('entity_id')->value;
    }
    elseif (!$this->get('entity_id_string')->isEmpty()) {
      return $this->get('entity_id_string')->value;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceEntityTypeId() {
    return $this->get('entity_type_id')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function getSourceEntity() {
    return \Drupal::entityTypeManager()
      ->getStorage($this->getSourceEntityTypeId())
      ->load($this->getSourceEntityId());
  }

  /**
   * {@inheritdoc}
   */
  public function getUserId() {
    return $this->get('uid')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function getUser() {
    return $this->get('uid')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields[$entity_type->getKey('id')]->setSetting('size', 'big');

    $fields['workflow_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Workflow'))
      ->setDescription(t('The workflow.'))
      ->setSetting('target_type', 'workflow')
      ->setStorageRequired(TRUE);

    $fields['workspace_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Workspace'))
      ->setDescription(t('The workspace.'))
      ->setSetting('target_type', 'workspace')
      ->setStorageRequired(TRUE);

    $fields['entity_type_id'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Source entity type ID'))
      ->setDescription(t('The source entity type ID.'))
      ->setStorageRequired(TRUE);

    $fields['entity_id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Source entity ID'))
      ->setDescription(t('The source entity ID.'));

    $fields['entity_id_string'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Source entity string ID'))
      ->setDescription(t('The source entity string ID.'));

    $fields['entity_revision_id'] = BaseFieldDefinition::create('integer')
      ->setLabel(t('Source entity revision ID'))
      ->setDescription(t('The source entity revision ID.'));

    $fields['state'] = BaseFieldDefinition::create('string')
      ->setLabel(t('State'))
      ->setDescription(t('The new state.'));

    $fields['transition'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Transition'))
      ->setDescription(t('The transition.'));

    $fields['previous_state'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Previous state'))
      ->setDescription(t('The previous state.'));

    $fields['uid'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('User'))
      ->setDescription(t('The user for the transition log.'))
      ->setSetting('target_type', 'user')
      ->setDefaultValueCallback('Drupal\user\EntityOwnerTrait::getDefaultEntityOwner')
      ->setStorageRequired(TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time when the transition log was created.'))
      ->setStorageRequired(TRUE);

    $fields['log'] = BaseFieldDefinition::create('string_long')
      ->setLabel(t('Transition log message'))
      ->setDescription(t('Briefly describe the changes you have made.'))
      ->setDefaultValue('');

    return $fields;
  }

}
