<?php

namespace Drupal\entity_workflow;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeBase;
use Drupal\workflows\WorkflowInterface;
use Drupal\workspaces\WorkspaceInterface;
use Drupal\workspaces\WorkspaceManagerInterface;

class EntityWorkflowInfo {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type bundle information manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityBundleInfo;

  /**
   * The workspace manager.
   *
   * @var \Drupal\workspaces\WorkspaceManagerInterface
   */
  protected $workspaceManager;

  /**
   * Creates a EntityWorkflowInfo object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_bundle_info
   *   The entity type bundle info service.
   * @param \Drupal\workspaces\WorkspaceManagerInterface $workspace_manager
   *   The workspace manager.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $entity_bundle_info, WorkspaceManagerInterface $workspace_manager) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityBundleInfo = $entity_bundle_info;
    $this->workspaceManager = $workspace_manager;
  }

  /**
   * Determine if an entity is actually in the workflow.
   *
   * This is only true if the entity is part of the workspace and thus has a
   * state.
   *
   * @param \Drupal\Core\Entity\ContentEntityInterface $entity
   *   The entity.
   * @param string $workflow_id
   *   The workflow to check.
   * @param \Drupal\workspaces\WorkspaceInterface|null $workspace
   *   (optional) The workspace to check, or the current workspace if not
   *   specified.
   *
   * @return bool
   *   TRUE if the entity is in the workflow, FALSE otherwise.
   */
  public function isEntityInWorkflow(ContentEntityInterface $entity, $workflow_id, WorkspaceInterface $workspace = NULL) {
    // Entity workflows are tied to workspaces.
    if (!$this->workspaceManager->hasActiveWorkspace()) {
      return FALSE;
    }

    $workflows_info = $this->getWorkflowsInfoForEntityTypeAndBundle($entity->getEntityTypeId(), $entity->bundle());
    if (!isset($workflows_info[$workflow_id])) {
      return FALSE;
    }

    // Workspaces themselves should always be considered as part of a workflow.
    if ($entity->getEntityTypeId() === 'workspace') {
      return TRUE;
    }

    $workspace = $workspace ?? $this->workspaceManager->getActiveWorkspace();
    $field_name = $entity->getEntityType()->getRevisionMetadataKey('workspace');
    return $entity->get($field_name)->target_id == $workspace->id();
  }

  /**
   * Determines whether an entity type is supported by at least one workflow.
   *
   * @param string $entity_type_id
   *   The entity type ID.
   *
   * @return bool
   *   TRUE if the entity type is supported by at least one workflow, FALSE
   *   otherwise.
   */
  public function isEntityTypeSupported($entity_type_id) {
    return !empty($this->getWorkflowsInfoForEntityType($entity_type_id));
  }

  /**
   * @param string $entity_type_id
   *   The entity type ID.
   *
   * @return array
   *   The array of workflow labels, keyed by their workflow ID.
   */
  public function getWorkflowsInfoForEntityType($entity_type_id) {
    $bundles = $this->entityBundleInfo->getBundleInfo($entity_type_id);
    $entity_workflows = array_column($bundles, 'entity_workflows');

    return $entity_workflows ? reset($entity_workflows) : [];
  }

  /**
   * @param string $entity_type_id
   *   The entity type ID.
   * @param string $bundle_id
   *   The bundle ID.
   *
   * @return array
   *   An array of workflow labels, keyed by their workflow ID.
   */
  public function getWorkflowsInfoForEntityTypeAndBundle($entity_type_id, $bundle_id) {
    $bundle = $this->entityBundleInfo->getBundleInfo($entity_type_id)[$bundle_id];

    return $bundle['entity_workflows'] ?? [];
  }

  /**
   * @param string $entity_type_id
   *   The entity type ID.
   *
   * @return \Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeInterface[]
   *   An array of entity workflow plugins, keyed by their workflow entity ID.
   */
  public function getWorkflowPluginsForEntityType($entity_type_id) {
    $workflow_ids = array_keys($this->getWorkflowsInfoForEntityType($entity_type_id));
    $workflows = $this->entityTypeManager->getStorage('workflow')->loadMultiple($workflow_ids);

    return array_map(function (WorkflowInterface $workflow) {
      return $workflow->getTypePlugin();
    }, $workflows);
  }

  /**
   * @param string $entity_type_id
   *   The entity type ID.
   * @param string $bundle_id
   *   The bundle ID.
   *
   * @return \Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeInterface[]
   *   An array of entity workflow plugins, keyed by their workflow entity ID.
   */
  public function getWorkflowPluginsForEntityTypeAndBundle($entity_type_id, $bundle_id) {
    $workflow_ids = array_keys($this->getWorkflowsInfoForEntityTypeAndBundle($entity_type_id, $bundle_id));
    $workflows = $this->entityTypeManager->getStorage('workflow')->loadMultiple($workflow_ids);

    return array_map(function (WorkflowInterface $workflow) {
      return $workflow->getTypePlugin();
    }, $workflows);
  }

  /**
   * @param \Drupal\workflows\WorkflowInterface $workflow
   *   A workflow entity.
   *
   * @return bool
   */
  public function isEntityWorkflow(WorkflowInterface $workflow) {
    return is_subclass_of($workflow->getTypePlugin()->getPluginDefinition()['class'], EntityWorkflowTypeBase::class);
  }

  /**
   * @return \Drupal\workflows\WorkflowInterface[]
   *   An array of workflow entities.
   */
  public function getWorkflowEntities() {
    $workflows = $this->entityTypeManager->getStorage('workflow')->loadMultiple();
    $workflows = array_filter($workflows, function (WorkflowInterface $workflow) {
      return $this->isEntityWorkflow($workflow);
    });

    return $workflows;
  }

}
