<?php

namespace Drupal\entity_workflow;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\workflows\Transition;
use Drupal\workflows\TransitionInterface;
use Drupal\workflows\WorkflowTypeInterface;

/**
 * A value object representing a workflow transition for Entity Workflow.
 */
class EntityWorkflowTransition extends Transition implements TransitionInterface {

  /**
   * A callback that will determine the 'to' state for this transition.
   *
   * @var string
   */
  protected $toStateCallback;

  /**
   * An array of arguments for the state callback.
   *
   * @var array
   */
  protected $toStateCallbackArguments = [];

  /**
   * The access callback for this transition.
   *
   * @var string
   */
  protected $accessCallback;

  /**
   * An array of arguments for the access callback.
   *
   * @var array
   */
  protected $accessCallbackArguments = [];

  /**
   * Whether the transition can log a message.
   *
   * @var bool
   */
  protected $log;

  /**
   * Whether the transition log message is required.
   *
   * @var bool
   */
  protected $logRequired;

  /**
   * The transition confirmation message.
   *
   * @var string
   */
  protected $confirmation;

  /**
   * Whether this transition should be provided as an action link.
   *
   * @var bool
   */
  protected $actionLink;

  /**
   * EntityWorkflowTransition constructor.
   */
  public function __construct(WorkflowTypeInterface $workflow, $id, array $configuration) {
    parent::__construct($workflow, $id, $configuration['label'], $configuration['from'], $configuration['to'], $configuration['weight']);

    $this->toStateCallback = $configuration['to_state_callback'] ?? NULL;
    $this->toStateCallbackArguments = $configuration['to_state_callback_arguments'] ?? [];
    $this->accessCallback = $configuration['access_callback'] ?? NULL;
    $this->accessCallbackArguments = $configuration['access_callback_arguments'] ?? [];
    $this->log = $configuration['transition_log'] ?? FALSE;
    $this->logRequired = $configuration['transition_log_required'] ?? FALSE;
    $this->confirmation = $configuration['transition_confirmation'] ?? NULL;
    $this->actionLink = $configuration['action_link'] ?? FALSE;
  }

  /**
   * {@inheritdoc}
   */
  public function to(ContentEntityInterface $entity = NULL) {
    // If the transition is configured to get its 'to' state from a callback,
    // invoke it now.
    if ($callback = $this->toStateCallback) {
      if (!function_exists($callback)) {
        throw new \InvalidArgumentException("The {$this->id} transition does not have a valid 'to' state callback.");
      }

      $args = [$entity, $this];
      if (!empty($this->toStateCallbackArguments)) {
        $args = array_merge($args, $this->toStateCallbackArguments);
      }

      $to_state_id = $callback(...$args);

      if (!$to_state_id) {
        throw new \InvalidArgumentException("The {$this->id} transition does not have a valid 'to' state.");
      }

      $this->toStateId = $to_state_id;
    }

    return parent::to();
  }

  /**
   * Whether the transition can log a message.
   *
   * @return bool
   */
  public function hasLog() {
    return $this->log;
  }

  /**
   * Whether the transition log message is required.
   *
   * @return bool
   */
  public function isLogRequired() {
    return $this->logRequired;
  }

  /**
   * The transition confirmation message.
   *
   * @return string
   */
  public function confirmationMessage() {
    return $this->confirmation;
  }

  /**
   * Whether this transition should be provided as an action link.
   *
   * @return bool
   */
  public function hasActionLink() {
    return TRUE;
    return $this->actionLink;
  }

}
