<?php

namespace Drupal\entity_workflow\FieldType;

use Drupal\Core\Session\AccountInterface;
use Drupal\entity_workflow\Entity\WorkflowTransitionLog;
use Drupal\workflows\Plugin\Field\FieldType\WorkflowStateItem;
use Drupal\workflows\StateInterface;
use Drupal\workflows\TransitionInterface;
use Drupal\workspaces\WorkspaceInterface;

/**
 * Defines a custom field item class for the 'workflow_state' field type.
 */
class EntityWorkflowStateItem extends WorkflowStateItem {

  /**
   * {@inheritdoc}
   */
  public function applyDefaultValue($notify = TRUE) {
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function applyTransition(TransitionInterface $transition, $log = '') {
    $this->setValue(['value' => $transition->to($this->getEntity())->id()]);
    $this->transition_id = $transition->id();
    $this->log = $log;
  }

  /**
   * {@inheritdoc}
   */
  public function applyTransitionById($transition_id, $log = '') {
    $transition = $this->getWorkflow()->getTypePlugin()->getTransition($transition_id);
    $this->applyTransition($transition, $log);
  }

  /**
   * {@inheritdoc}
   */
  public function postSave($update) {
    parent::postSave($update);

    // Record state history.
    if (isset($this->transition_id)) {
      $transition_log = WorkflowTransitionLog::createFromEntity(
        $this->getEntity(),
        $this->getWorkflow()->id(),
        $this->getOriginalStateId(),
        $this->value,
        $this->transition_id,
        $this->log
      );
      $transition_log->save();
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function invokeTransitionHook($phase) {
    $workflow = $this->getWorkflow();
    if ($workflow->getTypePlugin()->hasTransitionFromStateToState($this->originalValue, $this->value, $this->getEntity())) {
      $transition = $workflow->getTypePlugin()->getTransitionFromStateToState($this->originalValue, $this->value, $this->getEntity());

      // Invoke the hook.
      $entity = $this->getEntity();
      $field_name = $this->getFieldDefinition()->getName();
      \Drupal::moduleHandler()->invokeAll('workflows_' . $phase, [$workflow, $transition, $entity, $field_name]);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getStateLabel(WorkspaceInterface $workspace = NULL) {
    /** @var \Drupal\workspaces\WorkspaceManagerInterface $workspace_manager */
    $workspace_manager = \Drupal::service('workspaces.manager');
    if (!$workspace && $workspace_manager->hasActiveWorkspace()) {
      $workspace = $workspace_manager->getActiveWorkspace();
    }
    $state = $this->getWorkflow()->getTypePlugin()->getState($this->value);
    return $state->label($this->getEntity(), $workspace);
  }

  /**
   * {@inheritdoc}
   */
  public function getPossibleOptions(AccountInterface $account = NULL) {
    /** @var \Drupal\workspaces\WorkspaceManagerInterface $workspace_manager */
    $workspace_manager = \Drupal::service('workspaces.manager');

    $workflow = $this->getWorkflow();
    $state_labels = array_map(function (StateInterface $state) use ($workspace_manager) {
      return $state->label($this->getEntity(), $workspace_manager->getActiveWorkspace());
    }, $workflow->getTypePlugin()->getStates());

    return $state_labels;
  }

  /**
   * {@inheritdoc}
   */
  public function getSettableOptions(AccountInterface $account = NULL) {
    /** @var \Drupal\workspaces\WorkspaceManagerInterface $workspace_manager */
    $workspace_manager = \Drupal::service('workspaces.manager');

    $allowed_states = $this->getAllowedStates($this->value);
    $state_labels = array_map(function (StateInterface $state) use ($workspace_manager) {
      // @todo Temporary safeguard for no active workspace. Needs investigation.
      $active_workspace = $workspace_manager->getActiveWorkspace();
      if (!$active_workspace) {
        $active_workspace = NULL;
      }
      return $state->label($this->getEntity(), $active_workspace);
    }, $allowed_states);

    return $state_labels;
  }

}
