<?php

namespace Drupal\entity_workflow\Form;

use Drupal\Core\Entity\EntityTypeBundleInfoInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\entity_workflow\EntityWorkflowInfo;
use Drupal\entity_workflow\StateTransitionValidatorInterface;
use Drupal\workflows\WorkflowInterface;
use Drupal\workspaces\WorkspaceInterface;
use Drupal\workspaces\WorkspaceManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

class EntityWorkflowBulkTransitionForm extends FormBase {

  /**
   * The entity workflow information service.
   *
   * @var \Drupal\entity_workflow\EntityWorkflowInfo
   */
  protected $entityWorkflowInfo;

  /**
   * The state transition validation service.
   *
   * @var \Drupal\entity_workflow\StateTransitionValidatorInterface
   */
  protected $stateTransitionValidator;

  /**
   * The workspace manager.
   *
   * @var \Drupal\workspaces\WorkspaceManagerInterface
   */
  protected $workspaceManager;

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity type bundle info service.
   *
   * @var \Drupal\Core\Entity\EntityTypeBundleInfoInterface
   */
  protected $entityTypeBundleInfo;

  /**
   * @var \Drupal\workflows\WorkflowInterface
   */
  protected $workflow;

  /**
   * @var \Drupal\entity_workflow\WorkflowType\EntityWorkflowTypeInterface
   */
  protected $workflowPlugin;

  /**
   * Creates a EntityWorkflowForm object.
   *
   * @param \Drupal\entity_workflow\EntityWorkflowInfo $entity_workflow_info
   *   The entity workflow information service.
   * @param \Drupal\entity_workflow\StateTransitionValidatorInterface $state_transition_validation
   *   The state transition validation service.
   * @param \Drupal\workspaces\WorkspaceManagerInterface $workspace_manager
   *   The workspace manager.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityTypeBundleInfoInterface $entity_type_bundle_info
   *   The entity type bundle info service.
   */
  public function __construct(EntityWorkflowInfo $entity_workflow_info, StateTransitionValidatorInterface $state_transition_validation, WorkspaceManagerInterface $workspace_manager, EntityTypeManagerInterface $entity_type_manager, EntityTypeBundleInfoInterface $entity_type_bundle_info) {
    $this->entityWorkflowInfo = $entity_workflow_info;
    $this->stateTransitionValidator = $state_transition_validation;
    $this->workspaceManager = $workspace_manager;
    $this->entityTypeManager = $entity_type_manager;
    $this->entityTypeBundleInfo = $entity_type_bundle_info;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_workflow.info'),
      $container->get('entity_workflow.state_transition_validation'),
      $container->get('workspaces.manager'),
      $container->get('entity_type.manager'),
      $container->get('entity_type.bundle.info')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'entity_workflow_bulk_transition_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, WorkflowInterface $workflow = NULL, array $entities = [], WorkspaceInterface $workspace = NULL) {
    if (!$workflow instanceof WorkflowInterface) {
      throw new \InvalidArgumentException('The workflow argument is required.');
    }
    if (!$workspace instanceof WorkspaceInterface) {
      throw new \InvalidArgumentException('The workspace argument is required.');
    }
    $this->workflow = $workflow;
    $this->workflowPlugin = $this->workflow->getTypePlugin();

    $form['#title'] = $this->t('@label bulk workflow', ['@label' => $this->workflow->label()]);

    // Build the sortable table header.
    $header = [
      'title' => ['data' => $this->t('Title')],
      'type' => ['data' => $this->t('Type')],
      'state' => ['data' => $this->t('State')],
    ];

    // Create a list of transitions that use log messages so we can
    // apply #states and properly validate.
    $transitions = $this->workspaceManager->hasActiveWorkspace()
      ? ['--create-draft--' => $this->t('Add to site version')]
      : [];
    $transition_entities = [];
    $log_transitions = [];
    $log_required = [];

    foreach ($this->workflowPlugin->getTransitions() as $transition_id => $transition) {
      $transitions[$transition_id] = $transition->label();
      if ($transition->hasLog()) {
        $log_transitions[] = $transition_id;
        if ($transition->isLogRequired()) {
          $log_required[$transition_id] = $transition_id;
        }
      }
    }

    // Build the rows.
    $options = [];
    $entity_info_all = $this->entityTypeManager->getDefinitions();
    foreach ($entities as $entity_data => $entity) {
      [$entity_type_id, $entity_id] = explode('--', $entity_data, 2);

      if ($this->entityWorkflowInfo->isEntityInWorkflow($entity, $workflow->id(), $workspace)) {
        $state = entity_workflow_get_entity_state($entity, $workflow->id());
        $label = $this->workflowPlugin->getState($state)->label($entity, $workspace);
        // Figure out which transitions are valid and enable them for this entity.
        foreach (array_keys($transitions) as $transition_id) {
          if ($this->stateTransitionValidator->isTransitionValid($entity, $workflow, $transition_id, $this->currentUser(), $workspace)) {
            $transition_entities[$transition_id][$entity_data] = $entity_data;
          }
        }
      }
      else {
        $label = $this->t('Not in workflow');
        // Only the add to site version transition is valid for entities not
        // already in the site version.
        $transition_entities['--create-draft--'][$entity_data] = $entity_data;
      }

      $entity_type_string = $entity_info_all[$entity_type_id]->getLabel();
      if (count($this->entityTypeBundleInfo->getBundleInfo($entity_type_id)) > 1) {
        $entity_type_string .= ': ' . $this->entityTypeBundleInfo->getBundleInfo($entity_type_id)[$entity->bundle()]['label'];
      }

      $options[$entity_data] = [
        'title' => $entity->label(),
        'type' => $entity_type_string,
        'state' => $label,
      ];
    }

    // Remove any transitions that are not valid for our sent entities.
    // array_filter() only lets you filter on keys as of PHP 5.6 so we have
    // to do this the old fashioned way.
    foreach ($transitions as $transition_id => $transition_info) {
      if (!isset($transition_entities[$transition_id])) {
        unset($transitions[$transition_id]);
      }
    }

    $form['actions'] = [
      '#type' => 'container',
      '#attributes' => ['class' => ['container-inline']],
    ];

    $form['actions']['transition'] = [
      '#type' => 'select',
      '#options' => $transitions,
      '#empty_option' => $this->t('Choose a transition'),
      '#empty_value' => '',
    ];

    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Execute'),
    ];

    if (!empty($log_transitions)) {
      $form['log'] = [
        '#type' => 'textarea',
        '#title' => $this->t('Provide a log message'),
      ];

      foreach ($log_transitions as $transition_id) {
        $form['log']['#states']['visible'][':input[name="transition"]'][] = ['value' => $transition_id];
      }

      if (!empty($log_required)) {
        $form['#log_required'] = $log_required;
        foreach ($log_required as $transition_id) {
          $form['log']['#states']['required'][':input[name="transition"]'][] = ['value' => $transition_id];
        }
      }
    }

    $form['entities'] = [
      '#type' => 'tableselect',
      '#header' => $header,
      '#options' => $options,
      '#empty' => $this->t('No entities available.'),
      // Disable the js select all because it does not work right with
      // the variable checkboxes.
      '#js_select' => FALSE,
    ];

    $form['workspace_id'] = [
      '#type' => 'value',
      '#value' => $workspace->id(),
    ];

    $form['#attached']['library'][] = 'entity_workflow/bulk';
    $form['#attached']['drupalSettings']['entityWorkflowBulkTransitions'] = $transition_entities;

    // Changing the workflow state of an entity is allowed in a workspace.
    $form_state->set('workspace_safe', TRUE);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $transition_id = $form_state->getValue('transition');
    if (!$transition_id) {
      $form_state->setError($form['actions']['transition'], $this->t('You must select a transition.'));
      return;
    }

    if ($transition_id === '--create-draft--') {
      return;
    }

    $transition = $this->workflowPlugin->getTransition($transition_id);
    if ($transition->hasLog() && $transition->isLogRequired() && empty($form_state->getValue('log'))) {
      $form_state->setError($form['log'], $this->t('Log field is required'));
    }

    // If they entered a log message when it was visible and then switched
    // to something where the log message was not visible, remove it so it
    // does not get recorded.
    if (!$transition->hasLog()) {
      $form_state->setValue('log', '');
    }

    if (!array_filter($form_state->getValue('entities'))) {
      $form_state->setError($form['entities'], $this->t('You must select one or more entities to transition.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $transition_id = $form_state->getValue('transition');
    $this->workspaceManager->executeInWorkspace($form_state->getValue('workspace_id'), function () use ($form_state, $transition_id) {
      foreach (array_filter($form_state->getValue('entities')) as $entity_data) {
        list($entity_type_id, $entity_id) = explode('--', $entity_data, 2);
        $entity = $this->entityTypeManager->getStorage($entity_type_id)->load($entity_id);
        if ($transition_id === '--create-draft--') {
          $entity->save();
        }
        else {
          $workflow_field = entity_workflow_get_field($entity, $this->workflow->id());
          $workflow_field->applyTransitionById($transition_id, $form_state->getValue('log'));
          $entity->_entityWorkflowEnforceNoNewRevision = TRUE;
          $entity->save();
        }
      }
    });

    $this->messenger()->addMessage($this->t('The transition has been executed.'));
  }

}
