<?php

namespace Drupal\entity_workflow\Routing;

use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Routing\RouteSubscriberBase;
use Drupal\Core\Routing\RoutingEvents;
use Drupal\entity_workflow\EntityWorkflowInfo;
use Symfony\Component\Routing\Route;
use Symfony\Component\Routing\RouteCollection;

/**
 * Subscriber for Entity Workflow routes.
 */
class RouteSubscriber extends RouteSubscriberBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity workflow information service.
   *
   * @var \Drupal\entity_workflow\EntityWorkflowInfo
   */
  protected $entityWorkflowInfo;

  /**
   * Constructs a RouteSubscriber object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, EntityWorkflowInfo $entity_workflow_info) {
    $this->entityTypeManager = $entity_type_manager;
    $this->entityWorkflowInfo = $entity_workflow_info;
  }

  /**
   * {@inheritdoc}
   */
  protected function alterRoutes(RouteCollection $collection) {
    foreach ($this->entityTypeManager->getDefinitions() as $entity_type_id => $entity_type) {
      if ($this->entityWorkflowInfo->isEntityTypeSupported($entity_type_id)) {
        $base_path = NULL;
        if ($entity_type->hasLinkTemplate('canonical')) {
          $base_path = $entity_type->getLinkTemplate('canonical');
        }
        elseif ($entity_type->hasLinkTemplate('edit-form')) {
          $base_path = $entity_type->getLinkTemplate('edit-form');
        }

        if (!$base_path) {
          continue;
        }

        $parameters = [
          $entity_type_id => [
            'type' => "entity:$entity_type_id",
          ],
          'workflow' => [
            'type' => 'entity:workflow',
          ],
        ];

        // Add a route for the workflow form.
        $route = new Route($base_path . '/workflow/{workflow}');
        $route
          ->addDefaults([
            '_controller' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::workflow',
            'entity_type_id' => $entity_type_id,
          ])
          ->setRequirements([
            '_permission' => 'view workflow administration',
            '_custom_access' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::checkWorkflowAccess'
          ])
          ->setOption('parameters', $parameters)
          ->setOption('_admin_route', TRUE);
        $collection->add("entity.$entity_type_id.workflow", $route);

        // Add a route for the bulk workflow form. Workspaces don't need bulk
        // workflows for themselves.
        if ($entity_type_id !== 'workspace') {
          $route = new Route($base_path . '/bulk-workflow/{workflow}');
          $route
            ->addDefaults([
              '_controller' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::bulkWorkflow',
              'entity_type_id' => $entity_type_id,
            ])
            ->setRequirements([
              '_permission' => 'view workflow administration',
              '_custom_access' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::checkBulkWorkflowAccess'
            ])
            ->setOption('parameters', $parameters)
            ->setOption('_admin_route', TRUE);
          $collection->add("entity.$entity_type_id.bulk_workflow", $route);
        }

        // Add a route for the simple transition form.
        $route = new Route($base_path . '/workflow/{workflow}/{transition_id}');
        $route
          ->addDefaults([
            '_form' => '\Drupal\entity_workflow\Form\EntityWorkflowSimpleTransitionForm',
            'entity_type_id' => $entity_type_id,
            'transition_id' => '',
          ])
          ->setRequirements([
            '_permission' => 'view workflow administration',
            '_custom_access' => '\Drupal\entity_workflow\Form\EntityWorkflowSimpleTransitionForm::checkAccess'
          ])
          ->setOption('parameters', $parameters)
          ->setOption('_admin_route', TRUE);
        $collection->add("entity.$entity_type_id.workflow_transition", $route);

        // Disable all form operations (e.g. Edit, Layout Builder, Entityqueue)
        // in the Live workspace for supported entity types.
        $this->disableEntityForms($collection, $entity_type);
      }
    }

    // Add a route to workspaces for bulk operations.
    if ($entity_type = $this->entityTypeManager->getDefinition('workspace', FALSE)) {
      $route = new Route($entity_type->getLinkTemplate('canonical') . '/bulk-workflow/{workflow}');
      $route
        ->addDefaults([
          '_controller' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::bulkWorkflowWorkspace',
        ])
        ->setRequirements([
          '_permission' => 'view workflow administration',
          '_custom_access' => '\Drupal\entity_workflow\Controller\EntityWorkflowController::checkBulkWorkflowWorkspaceAccess'
        ])
        ->setOption('parameters', [
          'workspace' => [
            'type' => "entity:workspace",
          ],
          'workflow' => [
            'type' => 'entity:workflow',
          ],
        ])
        ->setOption('_admin_route', TRUE);
      $collection->add("entity.workspace.bulk_workflow", $route);
    }
  }

  /**
   * Disables various entity forms in the Live workspace.
   *
   * @param \Symfony\Component\Routing\RouteCollection $collection
   *   The route collection.
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type.
   */
  protected function disableEntityForms(RouteCollection $collection, EntityTypeInterface $entity_type) {
    if ($entity_type->id() === 'workspace') {
      return;
    }

    // Disable entity forms.
    foreach ($collection as $route) {
      $needs_active_workspace = FALSE;

      // Disable 'add' forms.
      if ($entity_create_requirement = $route->getRequirement('_entity_create_access')) {
        [$entity_type_id] = explode(':', $entity_create_requirement);
        if ($entity_type_id === $entity_type->id()) {
          $needs_active_workspace = TRUE;
        }
      }

      // Disable 'edit/delete' forms.
      if ($entity_form = $route->getDefault('_entity_form')) {
        [$entity_type_id] = explode('.', $entity_form, 2);
        if ($entity_type_id === $entity_type->id()) {
          $needs_active_workspace = TRUE;
        }
      }

      if ($needs_active_workspace) {
        $route->addRequirements(['_has_active_workspace' => 'true']);
      }
    }

    // Disable Layout Builder forms.
    $suffixes = ['view', 'discard_changes', 'revert'];
    foreach ($suffixes as $suffix) {
      if ($route = $collection->get("layout_builder.overrides.{$entity_type->id()}.$suffix")) {
        $route->addRequirements(['_has_active_workspace' => 'true']);
      }
    }

    // Disable Entityqueue forms.
    if ($route = $collection->get("entity.{$entity_type->id()}.entityqueue")) {
      $route->addRequirements(['_has_active_workspace' => 'true']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    $events = parent::getSubscribedEvents();
    // Ensure that we run after Entityqueue's route subscriber so that we can
    // disable those routes as well.
    $events[RoutingEvents::ALTER] = ['onAlterRoutes', -215];
    return $events;
  }

}
