<?php

namespace Drupal\entity_workflow;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\workflows\WorkflowInterface;
use Drupal\workspaces\WorkspaceInterface;
use Drupal\workspaces\WorkspaceManagerInterface;

/**
 * Validates whether a certain state transition is allowed.
 */
class StateTransitionValidator implements StateTransitionValidatorInterface {

  /**
   * The workspace manager.
   *
   * @var \Drupal\workspaces\WorkspaceManagerInterface
   */
  protected $workspaceManager;

  /**
   * Constructs a new StateTransitionValidator.
   *
   * @param \Drupal\workspaces\WorkspaceManagerInterface $workspace_manager
   *   The workspace manager.
   */
  public function __construct(WorkspaceManagerInterface $workspace_manager) {
    $this->workspaceManager = $workspace_manager;
  }

  /**
   * {@inheritdoc}
   */
  public function getValidTransitions(ContentEntityInterface $entity, WorkflowInterface $workflow, AccountInterface $user, WorkspaceInterface $workspace = NULL) {
    return array_filter($workflow->getTypePlugin()->getTransitions(), function ($transition) use ($entity, $workflow, $user, $workspace) {
      return $this->isTransitionValid($entity, $workflow, $transition->id(), $user, $workspace);
    });
  }

  /**
   * {@inheritdoc}
   */
  public function isTransitionValid(ContentEntityInterface $entity, WorkflowInterface $workflow, $transition_id, AccountInterface $user, WorkspaceInterface $workspace = NULL) {
    $transitions_info = $workflow->getTypePlugin()->getConfiguration()['transitions'];

    if (!isset($transitions_info[$transition_id])) {
      return FALSE;
    }

    // First, make sure we're in a workspace, otherwise no transition is
    // allowed.
    if (!$this->workspaceManager->hasActiveWorkspace()) {
      return FALSE;
    }

    $workspace = $workspace ?? $this->workspaceManager->getActiveWorkspace();
    // @todo Implement workspace states.
    //$states = cps_changeset_get_states();
    //if ($states[$changeset->status]['type'] != 'open') {
    //  return FALSE;
    //}

    // Second, test valid 'from' states if we have them.
    if (isset($transitions_info[$transition_id]['from'])) {
      $current_state = entity_workflow_get_entity_state($entity, $workflow->id());
      if (!in_array($current_state, $transitions_info[$transition_id]['from'], TRUE)) {
        return FALSE;
      }
    }

    // Skip self transitions automatically.
    if (isset($transitions_info[$transition_id]['to'])) {
      $current_state = entity_workflow_get_entity_state($entity, $workflow->id());
      if ($current_state == $transitions_info[$transition_id]['to']) {
        return FALSE;
      }
    }

    // Then get our callback.
    $callback = NULL;
    if (isset($transitions_info[$transition_id]['access_callback'])) {
      $callback = $transitions_info[$transition_id]['access_callback'];
    }

    // Assume no access callback means the transition is accessible.
    if (!function_exists($callback)) {
      return TRUE;
    }

    $args = [$entity, $workflow, $transition_id, $user, $workspace];
    if (isset($transitions_info[$transition_id]['access_arguments'])) {
      $args = array_merge($args, $transitions_info[$transition_id]['access_arguments']);
    }

    return $callback(...$args);
  }

}
